-- This is a demonstration of how to use Lua's string matching and pattern captures.

function love.load()
	--love._openConsole()
	love.window.setMode(1,1,{borderless=true})
	local String =	"I would like to say 'Hello hello world!' 2u all.\t"--\r\t\1"
	tmp = 			'1234567890123456789012345678901234567890123456789012345678'
	print(String)
	-- % itself represents an escape character for 'magic characters' and non-alphanumeric characters.
	a,b=String:find( '%.' )
	print('non_alphanumeric:(%.) ',a,b,String:sub(a,b) )-- Returns 48, 48.

	-- . represents any character.
	a,b=String:find( 'h.llo' ) -- Returns 28, 32
	print('find word "h llo":(h.llo)',a,b,String:sub(a,b))

	-- %a represents all letters.
	a,b=String:find( '%aello') -- Returns 22, 26
	print('upper case followed by "ello":(%aello)',a,b,String:sub(a,b))

	-- %c represents all control characters (\n, \t, etc.).
	a,b = String:find( '%c+' ) -- Returns 49, 52
	print('control characters:(%c+) ',a,b,String:sub(a,b))

	-- %d represents all digits.
	a,b = String:find( '%d' ) -- Returns 42, 42
	print('find first digit:(%d) ',a,b,String:sub(a,b))

	-- %l represents lower-case letters.
	a,b=String:find( '%le%ll%l' ) -- Returns 28, 32
	print('lower-case "ll" lower-case:(%le%ll%l)',a,b,String:sub(a,b))

	-- %p represents all punctuation characters.
	a,b=String:find( '%p' ) -- Returns 21, 21
	print('punctuation character:(%p)',a,b,String:sub(a,b))

	-- %s represents all space characters.
	a,b=String:find( '%swor' ) -- Returns 33, 36
	print('space followed by "wor":(%swor)',a,b,String:sub(a,b))

	-- %u represents all upper-case letters.
	a,b = String:find( '%ue' ) -- Returns 22, 23
	print('upper-case followed by "e":(%ue)',a,b,String:sub(a,b))

	-- %w represents all alpha-numeric characters.
	a,b=String:find( '%w' ) -- Returns 1, 1
	print('first upper-case character:(%w)',a,b,String:sub(a,b))

	-- %x represents hexadecimal digits.
	-- %z represents the character with representation 0

	-- [ and ] are for sets. Whichever one appears first is returned first.
	String:find( '[%p%d%l]' ) -- Returns 3, 3, since the lower-case letter appears first.
	-- You can also use - to do a range.
	String:find( '[ - ]' ) -- Returns 2, 2

	a,b=String:find( '[a-z]' ) -- Returns 3, 3
	print('first lowercase:([a-z])',a,b,String:sub(a,b))
	a,b=String:find( '[A-Z]' ) -- Returns 1, 1
	print('first upper-case:([A-Z])',a,b,String:sub(a,b))

	-- [^ ] represents the complement (opposite) of a set.
	a,b=String:find( '[^%l]' ) -- Returns 1, 1, since the first non lower-case letter is at 1, 1
	print('first non lower-case:([^%l])',a,b,String:sub(a,b))
	-- All classes (%) that are upper-case represent the opposite of the class.
	a,b=String:find( '%L' ) -- is the same as String:find( '%u' ).
	print('upper-case:(%L)',a,b,String:sub(a,b))

	-- When something is followed by a * that means it can appear 0 or more times.
	a,b=String:find( 'hee*llo' ) -- Returns 28, 32
	print('*:(hee*llo)',a,b,String:sub(a,b))

	-- When something is followed by a + that means it can appear 1 or more times.
	a,b=String:find( 'he+llo' ) -- Returns 28, 32
	print('+:(he+llo)',a,b,String:sub(a,b))
	String:find( 'hee+llo' ) -- Returns nil.
	a,b=String:find( 'w.+d' ) -- Returns 3, 38
	print("return 'w' thru last 'd': (w.+d')",a,b,String:sub(a,b))

	-- When something is followed by a - that means it can appear 1 or more times, but always gets the lowest.
	a,b=String:find( 'w.-d' ) -- Returns 3, 7
	print("return everything from 'w' thru 'd': (w.-d)",a,b,String:sub(a,b))

	-- When something is followed by a ? that means it can appear 0 to 1 times. Like *, but limits it to 1.
	a,b=String:find( 'alll?' ) -- Returns 45, 47
	print('find "all":(all?)',a,b,String:sub(a,b))

	-- %n, where n is a number 1-9, where the item must match a substring equal to the nth captured term.
	a,b,c=String:find( '(%w+) h%1' ) -- Returns 23, 32, ello
	print('((%w+) h%1',a,b,c,String:sub(a,b))

	-- %bxy where b is the class and x and y are captures.
	a,b=String:find( '%b\'\'' ) -- Returns 21, 40.
	print("captures quoted text(%b\'\')",a,b,String:sub(a,b))

	-- ^ represents the beginning of the string, $ represents the end, only at the beginning and end of a string, respectively.
	String:find( '^%l' ) -- Returns nil, since the beginning of the string does not have a lower-case letter.
	String:find( '%l$' ) -- Returns nil, since the end of the string is punctuation, not a lower-case letter.

	-- Using ( ) "captures" the item, and returns it as a third parameter.
	a,b,c=String:find( 'w(.)u' ) -- Returns 3, 5, o.
	print('captures enclosed character:(w(.)u)',a,b,c,String:sub(a,b))
end

function love.keypressed( key )
	if key == 'escape'	then love.event.quit() end
	end
function love.update( dt )
end
